@@ -146,6 +146,7 @@ VKAPI_ATTR VkResult VKAPI_CALL test_vkCreateInstance(const VkInstanceCreateInfo*
     VkLayerInstanceCreateInfo* chain_info = get_chain_info(pCreateInfo, VK_LAYER_LINK_INFO);
 
     PFN_vkGetInstanceProcAddr fpGetInstanceProcAddr = chain_info->u.pLayerInfo->pfnNextGetInstanceProcAddr;
+    PFN_vk_icdGetPhysicalDeviceProcAddr fpGetPhysicalDeviceProcAddr = chain_info->u.pLayerInfo->pfnNextGetPhysicalDeviceProcAddr;
     PFN_vkCreateInstance fpCreateInstance = (PFN_vkCreateInstance)fpGetInstanceProcAddr(NULL, "vkCreateInstance");
     if (fpCreateInstance == NULL) {
         return VK_ERROR_INITIALIZATION_FAILED;
@@ -161,15 +162,36 @@ VKAPI_ATTR VkResult VKAPI_CALL test_vkCreateInstance(const VkInstanceCreateInfo*
         return result;
     }
     layer.instance_handle = *pInstance;
-    layer.next_GetPhysicalDeviceProcAddr =
-        reinterpret_cast<PFN_GetPhysicalDeviceProcAddr>(fpGetInstanceProcAddr(*pInstance, "vk_layerGetPhysicalDeviceProcAddr"));
-
+    if (layer.use_gipa_GetPhysicalDeviceProcAddr) {
+        layer.next_GetPhysicalDeviceProcAddr =
+            reinterpret_cast<PFN_GetPhysicalDeviceProcAddr>(fpGetInstanceProcAddr(*pInstance, "vk_layerGetPhysicalDeviceProcAddr"));
+    } else {
+        layer.next_GetPhysicalDeviceProcAddr = fpGetPhysicalDeviceProcAddr;
+    }
     // Init layer's dispatch table using GetInstanceProcAddr of
     // next layer in the chain.
     layer_init_instance_dispatch_table(layer.instance_handle, &layer.instance_dispatch_table, fpGetInstanceProcAddr);
 
     if (layer.create_instance_callback) result = layer.create_instance_callback(layer);
 
+    for (auto& func : layer.custom_physical_device_interception_functions) {
+        auto next_func = layer.next_GetPhysicalDeviceProcAddr(*pInstance, func.name.c_str());
+        layer.custom_dispatch_functions.at(func.name.c_str()) = next_func;
+    }
+
+    for (auto& func : layer.custom_device_interception_functions) {
+        auto next_func = layer.next_vkGetInstanceProcAddr(*pInstance, func.name.c_str());
+        layer.custom_dispatch_functions.at(func.name.c_str()) = next_func;
+    }
+
+    if (layer.do_spurious_allocations_in_create_instance && pAllocator && pAllocator->pfnAllocation) {
+        layer.spurious_instance_memory_allocation =
+            pAllocator->pfnAllocation(pAllocator->pUserData, 100, 8, VK_SYSTEM_ALLOCATION_SCOPE_INSTANCE);
+        if (layer.spurious_instance_memory_allocation == nullptr) {
+            return VK_ERROR_OUT_OF_HOST_MEMORY;
+        }
+    }
+
     return result;
 }
 
@@ -179,6 +201,11 @@ VKAPI_ATTR VkResult VKAPI_CALL test_override_vkCreateInstance(const VkInstanceCr
 }
 
 VKAPI_ATTR void VKAPI_CALL test_vkDestroyInstance(VkInstance instance, const VkAllocationCallbacks* pAllocator) {
+    if (layer.spurious_instance_memory_allocation && pAllocator && pAllocator->pfnFree) {
+        pAllocator->pfnFree(pAllocator->pUserData, layer.spurious_instance_memory_allocation);
+        layer.spurious_instance_memory_allocation = nullptr;
+    }
+
     layer.instance_dispatch_table.DestroyInstance(instance, pAllocator);
 }
 
@@ -208,6 +235,11 @@ VKAPI_ATTR VkResult VKAPI_CALL test_vkCreateDevice(VkPhysicalDevice physicalDevi
     // initialize layer's dispatch table
     layer_init_device_dispatch_table(device.device_handle, &device.dispatch_table, fpGetDeviceProcAddr);
 
+    for (auto& func : layer.custom_device_interception_functions) {
+        auto next_func = layer.next_vkGetDeviceProcAddr(*pDevice, func.name.c_str());
+        layer.custom_dispatch_functions.at(func.name.c_str()) = next_func;
+    }
+
     if (layer.create_device_callback) {
         result = layer.create_device_callback(layer);
     }
@@ -215,6 +247,15 @@ VKAPI_ATTR VkResult VKAPI_CALL test_vkCreateDevice(VkPhysicalDevice physicalDevi
     // Need to add the created devices to the list so it can be freed
     layer.created_devices.push_back(device);
 
+    if (layer.do_spurious_allocations_in_create_device && pAllocator && pAllocator->pfnAllocation) {
+        void* allocation = pAllocator->pfnAllocation(pAllocator->pUserData, 110, 8, VK_SYSTEM_ALLOCATION_SCOPE_DEVICE);
+        if (allocation == nullptr) {
+            return VK_ERROR_OUT_OF_HOST_MEMORY;
+        } else {
+            layer.spurious_device_memory_allocations.push_back({allocation, device.device_handle});
+        }
+    }
+
     return result;
 }
 
@@ -406,6 +447,16 @@ VKAPI_ATTR VkResult VKAPI_CALL test_vkEnumeratePhysicalDeviceGroups(
 // device functions
 
 VKAPI_ATTR void VKAPI_CALL test_vkDestroyDevice(VkDevice device, const VkAllocationCallbacks* pAllocator) {
+    for (uint32_t i = 0; i < layer.spurious_device_memory_allocations.size();) {
+        auto& allocation = layer.spurious_device_memory_allocations[i];
+        if (allocation.device == device && pAllocator && pAllocator->pfnFree) {
+            pAllocator->pfnFree(pAllocator->pUserData, allocation.allocation);
+            layer.spurious_device_memory_allocations.erase(layer.spurious_device_memory_allocations.begin() + i);
+        } else {
+            i++;
+        }
+    }
+
     for (auto& created_device : layer.created_devices) {
         if (created_device.device_handle == device) {
             created_device.dispatch_table.DestroyDevice(device, pAllocator);
@@ -413,11 +464,37 @@ VKAPI_ATTR void VKAPI_CALL test_vkDestroyDevice(VkDevice device, const VkAllocat
         }
     }
 }
+// forward declarations needed for trampolines
+#if TEST_LAYER_EXPORT_GET_PHYSICAL_DEVICE_PROC_ADDR
+extern "C" {
+FRAMEWORK_EXPORT VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL vk_layerGetPhysicalDeviceProcAddr(VkInstance instance, const char* pName);
+}
+#endif
 
 // trampolines
+VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_device_func(VkDevice device, const char* pName);
+VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_device_func_impl(VkDevice device, const char* pName) {
+    if (string_eq(pName, "vkGetDeviceProcAddr")) return to_vkVoidFunction(get_device_func);
+    if (string_eq(pName, "vkDestroyDevice")) return to_vkVoidFunction(test_vkDestroyDevice);
+
+    for (auto& func : layer.custom_device_interception_functions) {
+        if (func.name == pName) {
+            return to_vkVoidFunction(func.function);
+        }
+    }
+
+    for (auto& func : layer.custom_device_implementation_functions) {
+        if (func.name == pName) {
+            return to_vkVoidFunction(func.function);
+        }
+    }
+
+    return nullptr;
+}
 
 VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_device_func(VkDevice device, const char* pName) {
-    if (string_eq(pName, "vkDestroyDevice")) return to_vkVoidFunction(test_vkDestroyDevice);
+    PFN_vkVoidFunction ret_dev = get_device_func_impl(device, pName);
+    if (ret_dev != nullptr) return ret_dev;
 
     return layer.next_vkGetDeviceProcAddr(device, pName);
 }
@@ -430,15 +507,25 @@ VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_physical_device_func(VkInstance ins
     if (string_eq(pName, "vkEnumeratePhysicalDeviceGroups")) return (PFN_vkVoidFunction)test_vkEnumeratePhysicalDeviceGroups;
     if (string_eq(pName, "vkGetPhysicalDeviceProperties")) return (PFN_vkVoidFunction)test_vkGetPhysicalDeviceProperties;
 
-    for (auto& func : layer.custom_physical_device_functions) {
+    for (auto& func : layer.custom_physical_device_interception_functions) {
         if (func.name == pName) {
             return to_vkVoidFunction(func.function);
         }
     }
+
+    for (auto& func : layer.custom_physical_device_implementation_functions) {
+        if (func.name == pName) {
+            return to_vkVoidFunction(func.function);
+        }
+    }
+
+#if TEST_LAYER_EXPORT_GET_PHYSICAL_DEVICE_PROC_ADDR
+    if (string_eq(pName, "vk_layerGetPhysicalDeviceProcAddr")) return to_vkVoidFunction(vk_layerGetPhysicalDeviceProcAddr);
+#endif
     return nullptr;
 }
-
-VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_instance_func(VkInstance instance, const char* pName) {
+VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_instance_func(VkInstance instance, const char* pName);
+VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_instance_func_impl(VkInstance instance, const char* pName) {
     if (pName == nullptr) return nullptr;
     if (string_eq(pName, "vkGetInstanceProcAddr")) return to_vkVoidFunction(get_instance_func);
     if (string_eq(pName, "vkEnumerateInstanceLayerProperties")) return to_vkVoidFunction(test_vkEnumerateInstanceLayerProperties);
@@ -453,6 +540,16 @@ VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_instance_func(VkInstance instance,
     PFN_vkVoidFunction ret_phys_dev = get_physical_device_func(instance, pName);
     if (ret_phys_dev != nullptr) return ret_phys_dev;
 
+    PFN_vkVoidFunction ret_dev = get_device_func_impl(nullptr, pName);
+    if (ret_dev != nullptr) return ret_dev;
+
+    return nullptr;
+}
+
+VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL get_instance_func(VkInstance instance, const char* pName) {
+    PFN_vkVoidFunction ret_dev = get_instance_func_impl(instance, pName);
+    if (ret_dev != nullptr) return ret_dev;
+
     return layer.next_vkGetInstanceProcAddr(instance, pName);
 }
 
@@ -612,4 +709,4 @@ vkNegotiateLoaderLayerInterfaceVersion(VkNegotiateLayerInterface* pVersionStruct
     return VK_ERROR_INITIALIZATION_FAILED;
 }
 #endif
-}  // extern "C"
\ No newline at end of file
+}  // extern "C"
