@@ -45,11 +45,11 @@ void print_error_message(LSTATUS status, const char* function_name, std::string
     LPVOID lpMsgBuf;
     DWORD dw = GetLastError();
 
-    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, dw,
-                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)&lpMsgBuf, 0, NULL);
+    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, nullptr, dw,
+                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), reinterpret_cast<LPTSTR>(&lpMsgBuf), 0, nullptr);
 
     std::cerr << function_name << " failed with " << win_api_error_str(status) << ": "
-              << std::string(static_cast<LPTSTR>(lpMsgBuf));
+              << std::string(reinterpret_cast<LPTSTR>(lpMsgBuf));
     if (optional_message != "") {
         std::cerr << " | " << optional_message;
     }
@@ -58,29 +58,26 @@ void print_error_message(LSTATUS status, const char* function_name, std::string
 }
 
 void set_env_var(std::string const& name, std::string const& value) {
-    BOOL ret = SetEnvironmentVariableA(name.c_str(), value.c_str());
+    BOOL ret = SetEnvironmentVariableW(widen(name).c_str(), widen(value).c_str());
     if (ret == 0) {
-        print_error_message(ERROR_SETENV_FAILED, "SetEnvironmentVariableA");
+        print_error_message(ERROR_SETENV_FAILED, "SetEnvironmentVariableW");
     }
 }
-void remove_env_var(std::string const& name) { SetEnvironmentVariableA(name.c_str(), nullptr); }
-#define ENV_VAR_BUFFER_SIZE 4096
+void remove_env_var(std::string const& name) { SetEnvironmentVariableW(widen(name).c_str(), nullptr); }
 std::string get_env_var(std::string const& name, bool report_failure) {
-    std::string value;
-    value.resize(ENV_VAR_BUFFER_SIZE);
-    DWORD ret = GetEnvironmentVariable(name.c_str(), (LPSTR)value.c_str(), ENV_VAR_BUFFER_SIZE);
-    if (0 == ret) {
-        if (report_failure) print_error_message(ERROR_ENVVAR_NOT_FOUND, "GetEnvironmentVariable");
-        return std::string();
-    } else if (ENV_VAR_BUFFER_SIZE < ret) {
-        if (report_failure) std::cerr << "Not enough space to write environment variable" << name << "\n";
-        return std::string();
-    } else {
-        value.resize(ret);
+    std::wstring name_utf16 = widen(name);
+    DWORD value_size = GetEnvironmentVariableW(name_utf16.c_str(), nullptr, 0);
+    if (0 == value_size) {
+        if (report_failure) print_error_message(ERROR_ENVVAR_NOT_FOUND, "GetEnvironmentVariableW");
+        return {};
     }
-    return value;
+    std::wstring value(value_size, L'\0');
+    if (GetEnvironmentVariableW(name_utf16.c_str(), &value[0], value_size) != value_size - 1) {
+        return {};
+    }
+    return narrow(value);
 }
-#elif defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)
+#elif defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) || defined(__OpenBSD__)
 
 void set_env_var(std::string const& name, std::string const& value) { setenv(name.c_str(), value.c_str(), 1); }
 void remove_env_var(std::string const& name) { unsetenv(name.c_str()); }
@@ -137,7 +134,12 @@ std::string ManifestICD::get_manifest_str() const {
     out += "    \"ICD\": {\n";
     out += "        \"library_path\": \"" + fs::fixup_backslashes_in_path(lib_path) + "\",\n";
     out += "        \"api_version\": \"" + version_to_string(api_version) + "\",\n";
-    out += "        \"is_portability_driver\": " + to_text(is_portability_driver) + "\n";
+    out += "        \"is_portability_driver\": " + to_text(is_portability_driver);
+    if (!library_arch.empty()) {
+        out += ",\n       \"library_arch\": \"" + library_arch + "\"\n";
+    } else {
+        out += "\n";
+    }
     out += "    }\n";
     out += "}\n";
     return out;
@@ -174,8 +176,11 @@ std::string ManifestLayer::LayerDescription::get_manifest_str() const {
     print_vector_of_strings(out, "component_layers", component_layers);
     print_vector_of_strings(out, "blacklisted_layers", blacklisted_layers);
     print_vector_of_strings(out, "override_paths", override_paths);
+    print_vector_of_strings(out, "app_keys", app_keys);
     print_list_of_t(out, "pre_instance_functions", pre_instance_functions);
-
+    if (!library_arch.empty()) {
+        out += ",\n\t\t\"library_arch\": \"" + library_arch + "\"";
+    }
     out += "\n\t}";
 
     return out;
@@ -219,7 +224,7 @@ std::string make_native(std::string const& in_path) {
         else
             out += c;
     }
-#elif defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)
+#elif defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) || defined(__OpenBSD__)
     for (size_t i = 0; i < in_path.size(); i++) {
         if (i + 1 < in_path.size() && in_path[i] == '\\' && in_path[i + 1] == '\\') {
             out += '/';
@@ -345,7 +350,7 @@ path& path::replace_filename(path const& replacement) {
 // internal implementation helper for per-platform creating & destroying folders
 int create_folder(path const& path) {
 #if defined(WIN32)
-    return _mkdir(path.c_str());
+    return _wmkdir(widen(path.str()).c_str());
 #else
     mkdir(path.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
     return 0;
@@ -354,25 +359,27 @@ int create_folder(path const& path) {
 
 int delete_folder_contents(path const& folder) {
 #if defined(WIN32)
-    if (INVALID_FILE_ATTRIBUTES == GetFileAttributes(folder.c_str()) && GetLastError() == ERROR_FILE_NOT_FOUND) {
+    std::wstring folder_utf16 = widen(folder.str());
+    if (INVALID_FILE_ATTRIBUTES == GetFileAttributesW(folder_utf16.c_str()) && GetLastError() == ERROR_FILE_NOT_FOUND) {
         // nothing to delete
         return 0;
     }
-    std::string search_path = folder.str() + "/*.*";
+    std::wstring search_path = folder_utf16 + L"/*.*";
     std::string s_p = folder.str() + "/";
-    WIN32_FIND_DATA fd;
-    HANDLE hFind = ::FindFirstFileA(search_path.c_str(), &fd);
+    WIN32_FIND_DATAW fd;
+    HANDLE hFind = ::FindFirstFileW(search_path.c_str(), &fd);
     if (hFind != INVALID_HANDLE_VALUE) {
         do {
+            std::string file_name_utf8 = narrow(fd.cFileName);
             if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
-                if (!string_eq(fd.cFileName, ".") && !string_eq(fd.cFileName, "..")) {
-                    delete_folder(s_p + fd.cFileName);
+                if (!string_eq(file_name_utf8.c_str(), ".") && !string_eq(file_name_utf8.c_str(), "..")) {
+                    delete_folder(s_p + file_name_utf8);
                 }
             } else {
-                std::string child_name = s_p + fd.cFileName;
-                DeleteFile(child_name.c_str());
+                std::string child_name = s_p + file_name_utf8;
+                DeleteFileW(widen(child_name).c_str());
             }
-        } while (::FindNextFile(hFind, &fd));
+        } while (::FindNextFileW(hFind, &fd));
         ::FindClose(hFind);
     }
     return 0;
@@ -409,46 +416,54 @@ int delete_folder(path const& folder) {
     int ret = delete_folder_contents(folder);
     if (ret != 0) return ret;
 #if defined(WIN32)
-    _rmdir(folder.c_str());
+    _wrmdir(widen(folder.str()).c_str());
     return 0;
 #else
     return rmdir(folder.c_str());
 #endif
 }
 
-FolderManager::FolderManager(path root_path, std::string name, DebugMode debug) : debug(debug), folder(root_path / name) {
+#if defined(WIN32)
+std::wstring native_path(const std::string& utf8) { return widen(utf8); }
+#else
+const std::string& native_path(const std::string& utf8) { return utf8; }
+#endif
+
+FolderManager::FolderManager(path root_path, std::string name) noexcept : folder(root_path / name) {
     delete_folder_contents(folder);
     create_folder(folder);
 }
-FolderManager::~FolderManager() {
+FolderManager::~FolderManager() noexcept {
+    if (folder.str().empty()) return;
     auto list_of_files_to_delete = files;
     // remove(file) modifies the files variable, copy the list before deleting it
     // Note: the allocation tests currently leak the loaded driver handles because in an OOM scenario the loader doesn't bother
     // removing those. Since this is in an OOM situation, it is a low priority to fix. It does have the effect that Windows will
     // be unable to delete the binaries that were leaked.
     for (auto& file : list_of_files_to_delete) {
-        if (debug >= DebugMode::log) std::cout << "Removing manifest " << file << " at " << (folder / file).str() << "\n";
-        if (debug != DebugMode::no_delete) {
-            remove(file);
-        }
-    }
-    if (debug != DebugMode::no_delete) {
-        delete_folder(folder);
-    }
-    if (debug >= DebugMode::log) {
-        std::cout << "Deleting folder " << folder.str() << "\n";
+        remove(file);
     }
+    delete_folder(folder);
 }
+FolderManager::FolderManager(FolderManager&& other) noexcept : folder(other.folder), files(other.files) {
+    other.folder.str().clear();
+}
+FolderManager& FolderManager::operator=(FolderManager&& other) noexcept {
+    folder = other.folder;
+    files = other.files;
+    other.folder.str().clear();
+    return *this;
+}
+
 path FolderManager::write_manifest(std::string const& name, std::string const& contents) {
     path out_path = folder / name;
     auto found = std::find(files.begin(), files.end(), name);
     if (found != files.end()) {
         std::cout << "Overwriting manifest " << name << ". Was this intended?\n";
     } else {
-        if (debug >= DebugMode::log) std::cout << "Creating manifest " << name << " at " << out_path.str() << "\n";
         files.emplace_back(name);
     }
-    auto file = std::ofstream(out_path.str(), std::ios_base::trunc | std::ios_base::out);
+    auto file = std::ofstream(native_path(out_path.str()), std::ios_base::trunc | std::ios_base::out);
     if (!file) {
         std::cerr << "Failed to create manifest " << name << " at " << out_path.str() << "\n";
         return out_path;
@@ -461,17 +476,15 @@ void FolderManager::remove(std::string const& name) {
     path out_path = folder / name;
     auto found = std::find(files.begin(), files.end(), name);
     if (found != files.end()) {
-        if (debug >= DebugMode::log) std::cout << "Removing file " << name << " at " << out_path.str() << "\n";
-        if (debug != DebugMode::no_delete) {
-            int rc = std::remove(out_path.c_str());
-            if (rc != 0 && debug >= DebugMode::log) {
-                std::cerr << "Failed to remove file " << name << " at " << out_path.str() << "\n";
-            }
-
-            files.erase(found);
+        int rc = std::remove(out_path.c_str());
+        if (rc != 0) {
+            std::cerr << "Failed to remove file " << name << " at " << out_path.str() << "\n";
         }
+
+        files.erase(found);
+
     } else {
-        if (debug >= DebugMode::log) std::cout << "Couldn't remove file " << name << " at " << out_path.str() << ".\n";
+        std::cout << "Couldn't remove file " << name << " at " << out_path.str() << ".\n";
     }
 }
 
@@ -480,19 +493,18 @@ path FolderManager::copy_file(path const& file, std::string const& new_name) {
     auto new_filepath = folder / new_name;
     auto found = std::find(files.begin(), files.end(), new_name);
     if (found != files.end()) {
-        if (debug >= DebugMode::log) std::cout << "File location already contains" << new_name << ". Is this a bug?\n";
+        std::cout << "File location already contains" << new_name << ". Is this a bug?\n";
     } else if (file.str() == new_filepath.str()) {
-        if (debug >= DebugMode::log) std::cout << "Trying to copy " << new_name << " into itself. Is this a bug?\n";
+        std::cout << "Trying to copy " << new_name << " into itself. Is this a bug?\n";
     } else {
-        if (debug >= DebugMode::log) std::cout << "Copying file" << file.str() << " to " << new_filepath.str() << "\n";
         files.emplace_back(new_name);
     }
-    std::ifstream src(file.str(), std::ios::binary);
+    std::ifstream src(native_path(file.str()), std::ios::binary);
     if (!src) {
         std::cerr << "Failed to create file " << file.str() << " for copying from\n";
         return new_filepath;
     }
-    std::ofstream dst(new_filepath.str(), std::ios::binary);
+    std::ofstream dst(native_path(new_filepath.str()), std::ios::binary);
     if (!dst) {
         std::cerr << "Failed to create file " << new_filepath.str() << " for copying to\n";
         return new_filepath;
@@ -675,4 +687,36 @@ VkDeviceCreateInfo* DeviceCreateInfo::get() noexcept {
     dev.queueCreateInfoCount = static_cast<uint32_t>(device_queue_infos.size());
     dev.pQueueCreateInfos = device_queue_infos.data();
     return &dev;
-}
\ No newline at end of file
+}
+
+#if defined(WIN32)
+std::string narrow(const std::wstring& utf16) {
+    if (utf16.empty()) {
+        return {};
+    }
+    int size = WideCharToMultiByte(CP_UTF8, 0, utf16.data(), static_cast<int>(utf16.size()), nullptr, 0, nullptr, nullptr);
+    if (size <= 0) {
+        return {};
+    }
+    std::string utf8(size, '\0');
+    if (WideCharToMultiByte(CP_UTF8, 0, utf16.data(), static_cast<int>(utf16.size()), &utf8[0], size, nullptr, nullptr) != size) {
+        return {};
+    }
+    return utf8;
+}
+
+std::wstring widen(const std::string& utf8) {
+    if (utf8.empty()) {
+        return {};
+    }
+    int size = MultiByteToWideChar(CP_UTF8, 0, utf8.data(), static_cast<int>(utf8.size()), nullptr, 0);
+    if (size <= 0) {
+        return {};
+    }
+    std::wstring utf16(size, L'\0');
+    if (MultiByteToWideChar(CP_UTF8, 0, utf8.data(), static_cast<int>(utf8.size()), &utf16[0], size) != size) {
+        return {};
+    }
+    return utf16;
+}
+#endif
